/*
  3 button OLED scene controller for MySensors

  Arduino based scene controller with an OLED display

  November 18, 2018

  This is my version of an in-wall scene controller with an OLED display that communicates with a MySensors 
  gateway.  The PCB that I designed for this (https://www.openhardware.io/view/546) can use one of two different 
  types of OLED displays.  Both the I2C and SPI versions of the display will work with the board with a few 
  jumper configurations.  This code is for the I2C version of the SSD1306 OLED display.  The board supports 
  9 different buttons that can be set up in a number of configurations.  This example code uses a 3 button 
  ON/Off (6 microswitches) configuration.

*/

// Enable debug prints
//#define MY_DEBUG
//#define AUTO
//#define DEBUG

// Enable and select radio type attached 
#define MY_RADIO_NRF24
//#define MY_RADIO_RFM69
//#define MY_RS485

//#define MY_REPEATER_FEATURE

#include <SPI.h>
#include <MySensors.h>  
#include <Wire.h>
#include <string.h>
//Uncomment the following lines if using the I2C version of the display
#include "SSD1306Ascii.h"
#include "SSD1306AsciiWire.h"
#include <avr/pgmspace.h>

#include <Keypad.h>

#define I2C_ADDRESS 0x3C
#define RST_PIN -1
SSD1306AsciiWire display;

int timer_count = 0;

bool metric = false;

#define CHILD_ID_ONOFF1       0
#define CHILD_ID_ONOFF2       1
#define CHILD_ID_ONOFF3       2
#define CHILD_ID_DISPLAY      3

//Define the number of rows and columns for the keypad
const byte ROWS = 3; 
const byte COLS = 2; 

//Map the keys for the keypad
char keys[ROWS][COLS] = {
  {'1','2'},
  {'3','4'},
  {'5','6'}
};
//Arduino pins for the rows and columns in the button matrix. Choose the pins based 
//on your chosen configuration of buttons

//    |  D8  |  D7  |  D6  
//----|------|------|-------
// D3 |  1   |  2   |  3
//----|------|------|-------
// D4 |  4   |  5   |  6
//----|------|------|-------
// D5 |  7   |  8   |  9
//    |      |      |
byte rowPins[ROWS] = { 4, 5, 6 };
byte colPins[COLS] = { 3, 7 }; 

char* rowDef[ROWS] = { "Table Lamps", "Ceiling Fan", "Ceiling Light" };

char disp1[] = "";
char disp2[] = "";
char disp3[] = "";
char disp4[] = "";
char disp5[] = "";
char disp[6][23]; // = { disp1, disp2, disp3, disp4, disp5 };

int pos = 0;
int fontsize = 1;

int tMax = 30000;
int switchTimer = 0;

// Create the Keypad
Keypad kpd = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

//Define the switch message transports. These will depend on the key configuration you
//choose and how you need the buttons to function
MyMessage onOff1(CHILD_ID_ONOFF1, V_STATUS);
MyMessage onOff2(CHILD_ID_ONOFF2, V_STATUS);
MyMessage onOff3(CHILD_ID_ONOFF3, V_STATUS);                                                                          
MyMessage msgDisplay(CHILD_ID_DISPLAY, V_TEXT);                                                                      
MyMessage msgDispLn1(CHILD_ID_DISPLAY, V_VAR1);                                                                      
MyMessage msgDispLn2(CHILD_ID_DISPLAY, V_VAR2);                                                                      
MyMessage msgDispLn3(CHILD_ID_DISPLAY, V_VAR3);                                                                      
MyMessage msgDispLn4(CHILD_ID_DISPLAY, V_VAR4);                                                                      
MyMessage msgDispLn5(CHILD_ID_DISPLAY, V_VAR5);   
           
/**
 * presentation - Presents the node children to the gateway
 */                                           
void presentation()  
{ 
  //Send the sketch version information to the gateway
  sendSketchInfo("OLEDSwitch", "1.0");

  //Register allse nsors to gw (they will be created as child devices)
  present(CHILD_ID_ONOFF1, S_BINARY);
  present(CHILD_ID_ONOFF2, S_BINARY);
  present(CHILD_ID_ONOFF3, S_BINARY);
  present(CHILD_ID_DISPLAY, S_INFO);
}

/**
 * setup - setup/initialization code goes here
 */ 
void setup() {
  // put your setup code here, to run once:
  #if RST_PIN >= 0
    display.begin(&Adafruit128x64, I2C_ADDRESS, RST_PIN);
  #else // RST_PIN >= 0
    display.begin(&Adafruit128x64, I2C_ADDRESS);
  #endif // RST_PIN >= 0
  
  Serial.begin(9600);
  display.clear();
  Serial.println("Starting...");
}

/**
 * loop - The main code loop
 */ 
void loop() {
  // put your main code here, to run repeatedly:
  scanKeys();

  updateDisplay();
}

/**
 * receive - Handles messages received from the gateway
 */ 
void receive( const MyMessage &message ) {
  switch (message.sensor) {
    case CHILD_ID_ONOFF1:
      send(onOff1.set(atoi(message.data)));
      break;
    case CHILD_ID_ONOFF2:
      send(onOff2.set(atoi(message.data)));
      break;
    case CHILD_ID_ONOFF3:
      send(onOff3.set(atoi(message.data)));
      break;
    case CHILD_ID_DISPLAY:
      int var = 0;
      if (message.type == V_VAR1) {
        strncpy(disp[1], (char*)message.data, 22);
      } else if (message.type == V_VAR2) {
        strncpy(disp[2], (char*)message.data, 22);
      } else if (message.type == V_VAR3) {
        strncpy(disp[3], (char*)message.data, 22);
      } else if (message.type == V_VAR4) {
        strncpy(disp[4], (char*)message.data, 22);
      } else if (message.type == V_VAR5) {
        strncpy(disp[5], (char*)message.data, 22);
      }
      
      break;
  }
}

/**
 * scanKeys - Scans the key matrix an processes any keypress events
 */ 
void scanKeys() {
  char key = kpd.getKey();
  if(key)  // Check for a valid key.
  {
    switch (key)
    {
      case '1': //Bottom switch off
        send(onOff1.set("0"));
        displaySwitch(rowDef[0], "Off");
        break;
      case '2': //Bottom switch on
        send(onOff1.set("1"));
        displaySwitch(rowDef[0], "On");
        break;
      case '3': //Middle switch off
        send(onOff2.set("0"));
        displaySwitch(rowDef[1], "Off");
        break;
      case '4': //Middle switch on
        send(onOff2.set("1"));
        displaySwitch(rowDef[1], "On");
        break;
      case '5': //Top switch off
        send(onOff3.set("0"));
        displaySwitch(rowDef[2], "Off");
        break;
      case '6': //Top switch on
        send(onOff3.set("1"));
        displaySwitch(rowDef[2], "On");
        break;
      default:
        Serial.println(key);
    }
  }
}

/**
 * updateDisplay - Handles the information sent to the OLED display
 */ 
void updateDisplay() {
  if (switchTimer > 0) {
    switchTimer --;
  } else if (switchTimer == 0) {
    display.clear();
    switchTimer = -1;
  } else {
    display.setFont(Adafruit5x7);
    display.home();
    for(int i = 1; i <= 5; i++) {
      if ( i == 1 || i == 5) {
        display.set1X();
      }else {
        display.set2X();
      }
      display.println(disp[i]);
    }
  }
}

/**
 * displaySwitch - Used to print out which switch was pressed on the OLED display
 */ 
void displaySwitch(char* name, char* state) {
  display.setFont(Adafruit5x7);
  display.clear();
  display.set1X();
  display.setCursor(65-(strlen(name)*3),3);
  display.println(name);
  display.set2X();
  display.setCursor(55-(strlen(state)*1.25),5);
  display.println(state);
  switchTimer = 15000;
}